/*
** Check options
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "netaudio.h"

typedef struct {
	char *entry;
	void (*function)();
} OPTIONS;

static void receive(int, int, char *[]);
static void send(int, int, char *[]);
static void broadcast(int, int, char *[]);
static void framesize(int, int, char *[]);
static void http(int, int, char *[]);
static void connections(int, int, char *[]);
static void verbose(int, int, char *[]);
static void quiet(int, int, char *[]);
static void version(int, int, char *[]);

OPTIONS options[] = {
	{ "-r", receive },	{ "--receive", receive },
	{ "-s", send },		{ "--send", send },
	{ "-b", broadcast },	{ "--broadcast", broadcast },
	{ "-f", framesize },	{ "--framesize", framesize },
	{ "-h", http },		{ "--http", http },
	{ "-c", connections },	{ "--connections", connections },
	{ "-v", verbose },	{ "--verbose", verbose },
	{ "-q", quiet },	{ "--quiet", quiet },
				{ "--version", version },
	{ NULL, NULL }
};

char *address, *bind_address;
int port, bind_port, mode, frame_size, is_verbose, is_quiet, is_http;

static void receive(int i, int argc, char *argv[])
{
    if (mode) {
	if (mode == MODE_SEND) {
	    if (!is_quiet)
		fprintf(stderr, "netaudio: overriding --send\n");
	} else if (mode == MODE_BROADCAST) {
	    if (!is_quiet)
		fprintf(stderr, "netaudio: overriding --broadcast\n");
	} else if (mode == MODE_RECEIVE) {
	    fprintf(stderr, "netaudio: error: --receive already specified.\n");
	    exit(-1);
	}
    }

    mode = MODE_RECEIVE;

    if (i + 1 < argc) {
	/* Check for address first. */
	if (argv[i + 1][0] != '-') {
	    char addr[600];
	    int pt = 0;

	    i++;

	    bzero(addr, 600);
	    sscanf(argv[i], "%[^:\r\n]:%d%*[^\r\n]\r\n", addr, &pt);

	    address = (char *) malloc(strlen(addr));
	    strcpy(address, addr);

	    if (pt)
		port = pt;
	    else
		port = 31337;

	    if (!is_quiet)
		fprintf(stderr, "netaudio: Using %s:%d for connections.\n",
		    address, port);
	}
    } else
	return;

    /* And check for the -a modifier to bind to address ports. */
    if (i + 1 < argc) {
	i++;

	if ((!strncasecmp(argv[i], "-a", 2)) ||
	    (!strncasecmp(argv[i], "--attach", 8))) {
	    char addr[600];
	    int pt = 0;

	    if (i + 1 > argc) {
		fprintf(stderr, "netaudio: --attach requires address argument.\n");
		exit(-1);
	    }

	    i++;

	    bzero(addr, 600);
	    sscanf(argv[i], "%[^:\r\n]:%d%*[^\r\n]\r\n", addr, &pt);

	    bind_address = (char *) malloc(strlen(addr));
	    strcpy(bind_address, addr);

	    if (pt)
		bind_port = pt;
	    else
		bind_port = 31337;

	    if (!is_quiet)
		fprintf(stderr, "netaudio: Using %s:%d for binding.\n",
		    bind_address, bind_port);
	}
    } else
	return;
}

static void send(int i, int argc, char *argv[])
{
    if (mode) {
	if (mode == MODE_RECEIVE) {
	    if (!is_quiet)
		fprintf(stderr, "netaudio: overriding --receive\n");
	} else if (mode == MODE_BROADCAST) {
	    if (!is_quiet)
		fprintf(stderr, "netaudio: overriding --broadcast\n");
	} else if (mode == MODE_SEND) {
	    fprintf(stderr, "netaudio: Error: --send already specified.\n");
	    exit(-1);
	}
    }

    mode = MODE_SEND;

    if (i + 1 >= argc) {
	fprintf(stderr, "netaudio: --send requires address argument.\n");
	exit(-1);
    } else {
	char addr[600];
	int pt = 0;

	i++;

	bzero(addr, 600);
	sscanf(argv[i], "%[^:\r\n]:%d%*[^\r\n]\r\n", addr, &pt);

	address = (char *) malloc(strlen(addr));
	strcpy(address, addr);

	if (pt)
	    port = pt;
	else
	    port = 31337;

	if (!is_quiet)
	    fprintf(stderr, "netaudio: Using %s:%d for sending.\n",
		address, port);
    }
}

static void broadcast(int i, int argc, char *argv[])
{
    if (mode) {
	if (mode == MODE_RECEIVE) {
	    if (!is_quiet)
		fprintf(stderr, "netaudio: overriding --receive\n");
	} else if (mode == MODE_SEND) {
	    if (!is_quiet)
		fprintf(stderr, "netaudio: overriding --send\n");
	} else if (mode == MODE_BROADCAST) {
	    fprintf(stderr, "netaudio: Error: --broadcast already specified.\n");
	    exit(-1);
	}
    }

    mode = MODE_BROADCAST;

    if (i + 1 < argc) {
	if (argv[i + 1][0] != '-') {
	    char addr[600];
	    int pt = 0;

	    i++;

	    bzero(addr, 600);
	    sscanf(argv[i], "%[^:\r\n]:%d%*[^\r\n]\r\n", addr, &pt);

	    address = (char *) malloc(strlen(addr));
	    strcpy(address, addr);

	    if (pt)
		port = pt;
	    else
		port = 31337;

	    if (!is_quiet)
		fprintf(stderr, "netaudio: Using %s:%d for binding.\n",
		    address, port);
	}
    } else
	return;
}

static void framesize(int i, int argc, char *argv[])
{
    if (i + 1 >= argc) {
	fprintf(stderr, "netaudio: --framesize requires an integer argument.\n");
	exit(-1);
    } else {
	i++;

	if (argv[i][0] == '-') {
	    fprintf(stderr, "netaudio: --framesize requires an integer argument.\n");
	    exit(-1);
	}

	frame_size = atoi(argv[i]);

	if (frame_size == 0)
	    frame_size = 33;

	if (!is_quiet)
	    fprintf(stderr, "netaudio: Using %d as the frame size.\n",
		frame_size);
    }
}

static void http(int i, int argc, char *argv[])
{
    if (i + 1 >= argc) {
	fprintf(stderr, "netaudio: --http requires a mime-type (string) argument.\n");
	exit(-1);
    }

    i++;

    if (argv[i][0] == '-') {
	fprintf(stderr, "netaudio: --http requires a mime-type (string) argument.\n");
	exit(-1);
    }

    printf("Content-Type: %s\n\n", argv[i]);
    fflush(stdout);

    is_http = 1;
}

static void connections(int i, int argc, char *argv[])
{
    if (!is_quiet)
	fprintf(stderr, "netaudio: hmmm ... --connections has no real use yet.\n");
}

static void verbose(int i, int argc, char *argv[])
{
    fprintf(stderr, "netaudio: verbose mode.\n");
    fprintf(stderr, "netaudio: okay, I'll be REALLY talkative!\n");

    is_verbose = 1;
}

static void quiet(int i, int argc, char *argv[])
{
    is_quiet = 1;
}

static void version(int i, int argc, char *argv[])
{
    fprintf(stderr, "This is NetAudio %s\n" \
		    "by Ken Hollis <khollis@bitgate.com>\n\n", VERSION);
    fprintf(stderr, "See http://www.bitgate.com/netaudio/ for more details.\n");
}

void check_options(int argc, char *argv[])
{
    int i, x;

    for (i = 1; i < argc; i++) {
	for (x = 0; options[x].entry != NULL; x++) {
	    if (!strncasecmp(argv[i], options[x].entry, strlen(argv[i]))) {
		options[x].function(i, argc, argv);
	    }
	}
    }
}
